@@ -5,32 +5,29 @@
 #include <cstring>
 #include <sys/stat.h>
 #include <glib/gi18n.h>
+#include <algorithm>
 
 #ifdef _WIN32
 #  include <gdk/gdkwin32.h>
 #  include "win32/intl.h"
 #endif
 
-#include "stardict.h"
 #include "conf.h"
+#include "lib.h"
 #include "utils.h"
+#include "file.hpp"
 
 #include "dictmanagedlg.h"
 
-DictManageDlg::DictManageDlg()
+DictManageDlg::DictManageDlg(GtkWindow *pw, GdkPixbuf *di,  GdkPixbuf *tdi) :
+	parent_win(pw), dicts_icon(di), tree_dicts_icon(tdi), window(NULL)
 {
-	window = NULL;
 }
 
 void DictManageDlg::response_handler (GtkDialog *dialog, gint res_id, DictManageDlg *oDictManageDlg)
 {
-	switch (res_id) {
-		case GTK_RESPONSE_HELP:
+	if (res_id==GTK_RESPONSE_HELP)
     show_help("stardict-dictmanage");
-			break;
-		default:
-			oDictManageDlg->Close();
-	}
 }
 
 void DictManageDlg::on_wazard_button_toggled(GtkToggleButton *button, DictManageDlg *oDictManageDlg)
@@ -45,125 +42,67 @@ void DictManageDlg::on_appendix_button_toggled(GtkToggleButton *button, DictMana
 		gtk_notebook_set_current_page(GTK_NOTEBOOK(oDictManageDlg->notebook), 1);
 }
 
-void DictManageDlg::load_dir(gboolean istreedict, const gchar *dirname, const GSList *order_list, const GSList *disable_list, GtkListStore *model)
-{
-	GDir *dir = g_dir_open(dirname, 0, NULL);	
-  if (dir) {
-		GtkTreeIter iter;
-		const gchar *filename;	
-		gchar fullfilename[256];
-		gboolean loaded;
-		const GSList *tmplist1,*tmplist2;
-		gboolean disabled;
-    DictInfo dict_info;
-		while ((filename = g_dir_read_name(dir))!=NULL) {
-			sprintf(fullfilename, "%s" G_DIR_SEPARATOR_S "%s", dirname, filename);
-			if (g_file_test(fullfilename, G_FILE_TEST_IS_DIR)) {
-				load_dir(istreedict, fullfilename, order_list, disable_list, model);
-      } else if (g_str_has_suffix(filename,".ifo")) {
-				tmplist1 = order_list;
-				loaded = false;
-				while (tmplist1) {
-					if (strcmp((gchar *)(tmplist1->data), fullfilename) == 0) {
-						loaded = true;
-						break;
-					}
-					tmplist1 = g_slist_next(tmplist1);
-				}
-				if (loaded)
-					continue;
-	if (dict_info.load_from_ifo_file(fullfilename, istreedict)) {
-					tmplist2 = disable_list;
-					disabled = false;
-					while (tmplist2) {
-						if (strcmp((gchar *)(tmplist2->data), fullfilename) == 0) {
-							disabled = true;
-							break;
-						}
-						tmplist2 = g_slist_next(tmplist2);
-					}
-					gtk_list_store_append (model, &iter);				
-	  gtk_list_store_set (model, &iter, 0, !disabled, 
-			      1, dict_info.bookname.c_str(), 
-			      2, dict_info.wordcount, 
-			      3, dict_info.author.c_str(), 
-			      4, dict_info.email.c_str(), 
-			      5, dict_info.website.c_str(), 
-			      6, dict_info.description.c_str(), 
-			      7, dict_info.date.c_str(), 
-			      8, fullfilename, 9, true, -1);
-				}
-			}
+class GetInfo {
+public:
+	GetInfo(GtkListStore *model_, bool istreedict_):
+		model(model_), istreedict(istreedict_) {}
+	void operator()(const std::string& url, bool disable) {
+		DictInfo dictinfo;
+		if (dictinfo.load_from_ifo_file(url.c_str(), istreedict)) {
+			GtkTreeIter iter;
+			gtk_list_store_append(model, &iter);
+			gtk_list_store_set(model, &iter, 
+												 0, !disable, 
+												 1, dictinfo.bookname.c_str(), 
+												 2, dictinfo.wordcount, 
+												 3, dictinfo.author.c_str(), 
+												 4, dictinfo.email.c_str(), 
+												 5, dictinfo.website.c_str(),
+												 6, dictinfo.description.c_str(), 
+												 7, dictinfo.date.c_str(), 
+												 8, url.c_str(), 
+												 9, TRUE, //for editable.
+												 -1);
 		}
-		g_dir_close(dir);
 	}
+private:
+	GtkListStore *model;
+	bool istreedict;
+};
+
+static GtkTreeModel* __create_dict_tree_model(const std::list<std::string> &dirs,
+																							const std::list<std::string>& order_list,
+																							const std::list<std::string>& disable_list, bool istreedict)
+{
+	GtkListStore *model = 
+		gtk_list_store_new(10, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, 
+											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+											 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, 
+											 G_TYPE_BOOLEAN);
+	for_each_file(dirs, ".ifo", order_list, disable_list, 
+								GetInfo(model, istreedict));
+	
+	return GTK_TREE_MODEL(model);
 }
 
-GtkTreeModel* DictManageDlg::create_dict_tree_model (gboolean istreedict)
+GtkTreeModel* DictManageDlg::create_dict_tree_model(bool istreedict)
 {
-	GtkListStore *model;
-
-	model = gtk_list_store_new (10, G_TYPE_BOOLEAN, G_TYPE_STRING, G_TYPE_LONG, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
-
-  const GSList *order_list, *disable_list;
 	if (istreedict) {
-    order_list=conf->get_treedict_order_list();
-    disable_list=conf->get_treedict_disable_list();
-  } else {
-    order_list=conf->get_dict_order_list();
-    disable_list=conf->get_dict_disable_list();    
-	}
+		return __create_dict_tree_model(
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_dirs_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_order_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/treedict_disable_list"),
+																		istreedict
+																		);
+	} else {
+		return __create_dict_tree_model(
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_dirs_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_order_list"),
+																		conf->get_strlist("/apps/stardict/manage_dictionaries/dict_disable_list"),
+																		istreedict
+																		);
+	}		
 	
-	
-	GtkTreeIter iter;
-	
-	gchar *ifofilename;
-  const GSList *tmplist1,*tmplist2;
-	gboolean disabled;
-	tmplist1 = order_list;	
-	while (tmplist1) {
-		ifofilename = (gchar *)(tmplist1->data);
-		tmplist1 = g_slist_next(tmplist1);
-    DictInfo dict_info;
-		if (g_file_test(ifofilename, G_FILE_TEST_EXISTS)) {
-      if (dict_info.load_from_ifo_file(ifofilename, istreedict)) {
-				tmplist2 = disable_list;
-				disabled = false;
-				while (tmplist2) {
-					if (strcmp((gchar *)(tmplist2->data), ifofilename) == 0) {
-						disabled = true;
-						break;
-					}
-					tmplist2 = g_slist_next(tmplist2);
-				}
-				gtk_list_store_append (model, &iter);				
-	gtk_list_store_set (model, &iter, 0, !disabled, 
-			    1, dict_info.bookname.c_str(), 
-			    2, dict_info.wordcount, 
-			    3, dict_info.author.c_str(), 
-			    4, dict_info.email.c_str(), 
-			    5, dict_info.website.c_str(),
-			    6, dict_info.description.c_str(), 
-			    7, dict_info.date.c_str(), 
-			    8, ifofilename, 9, true, -1);
-			}
-		}
-	}
-
-#ifdef _WIN32
-#else
-	gchar home_dir[256];	
-	if (istreedict)
-		sprintf(home_dir, "%s/.stardict/treedict", g_get_home_dir());
-	else
-		sprintf(home_dir, "%s/.stardict/dic", g_get_home_dir());
-	load_dir(istreedict, home_dir, order_list, disable_list, model);
-#endif
-	std::string filename(gStarDictDataDir+G_DIR_SEPARATOR);
-	filename += (istreedict) ? "treedict" : "dic";
-
-	load_dir(istreedict, filename.c_str(), order_list, disable_list, model);
-	return GTK_TREE_MODEL (model);	
 }
 
 void DictManageDlg::on_dict_enable_toggled (GtkCellRendererToggle *cell, gchar *path_str, DictManageDlg *oDictManageDlg)
@@ -184,18 +123,19 @@ void DictManageDlg::on_dict_enable_toggled (GtkCellRendererToggle *cell, gchar *
 
 	gboolean have_iter;
 	gchar *filename;
-	GSList *disable_list = NULL;
+	std::list<std::string> disable_list;
 	
 	have_iter = gtk_tree_model_get_iter_first(model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (model, &iter, 0, &enable, -1);
 		if (!enable) {
 			gtk_tree_model_get (model, &iter, 8, &filename, -1);
-			disable_list = g_slist_append(disable_list, filename);
+			disable_list.push_back(filename);
+			g_free(filename);
 		}
 		have_iter = gtk_tree_model_iter_next(model, &iter);
 	}
-	conf->set_dict_disable_list(disable_list);
+	conf->set_strlist("/apps/stardict/manage_dictionaries/dict_disable_list", disable_list);
 }
 
 void DictManageDlg::on_treedict_enable_toggled (GtkCellRendererToggle *cell, gchar *path_str, DictManageDlg *oDictManageDlg)
@@ -216,18 +156,19 @@ void DictManageDlg::on_treedict_enable_toggled (GtkCellRendererToggle *cell, gch
 
 	gboolean have_iter;
 	gchar *filename;
-	GSList *disable_list = NULL;
+	std::list<std::string> disable_list;
 	
 	have_iter = gtk_tree_model_get_iter_first(model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (model, &iter, 0, &enable, -1);
 		if (!enable) {
 			gtk_tree_model_get (model, &iter, 8, &filename, -1);
-			disable_list = g_slist_append(disable_list, filename);
+			disable_list.push_back(filename);
+			g_free(filename);
 		}
 		have_iter = gtk_tree_model_iter_next(model, &iter);
 	}
-	conf->set_treedict_disable_list(disable_list);
+	conf->set_strlist("/apps/stardict/manage_dictionaries/treedict_disable_list", disable_list);
 }
 
 void DictManageDlg::drag_data_get_cb(GtkWidget *widget, GdkDragContext *ctx, GtkSelectionData *data, guint info, guint time, DictManageDlg *oDictManageDlg)
@@ -395,12 +336,12 @@ GtkWidget *DictManageDlg::create_dict_tree(gboolean istreedict)
 	return sw;
 }
 
-void DictManageDlg::write_order_list(gboolean istreedict)
+void DictManageDlg::write_order_list(bool istreedict)
 {
 	GtkTreeIter iter;
 	gboolean have_iter;
 	gchar *filename;
-	GSList *order_list = NULL;
+	std::list<std::string> order_list;
 	GtkTreeModel *now_tree_model;
 	
 	if (istreedict)
@@ -411,14 +352,15 @@ void DictManageDlg::write_order_list(gboolean istreedict)
 	have_iter = gtk_tree_model_get_iter_first(now_tree_model, &iter);
 	while (have_iter) {
 		gtk_tree_model_get (now_tree_model, &iter, 8, &filename, -1);
-		order_list = g_slist_append(order_list, filename);
+		order_list.push_back(filename);
+		g_free(filename);
 		have_iter = gtk_tree_model_iter_next(now_tree_model, &iter);
 	}
 	
 	if (istreedict)
-	  conf->set_treedict_order_list(order_list);
+	  conf->set_strlist("/apps/stardict/manage_dictionaries/treedict_order_list", order_list);
 	else
-	  conf->set_dict_order_list(order_list);
+	  conf->set_strlist("/apps/stardict/manage_dictionaries/dict_order_list", order_list);
 }
 
 void DictManageDlg::on_move_top_button_clicked(GtkWidget *widget, DictManageDlg *oDictManageDlg)
@@ -561,91 +503,87 @@ GtkWidget *DictManageDlg::create_buttons()
 
 void DictManageDlg::Show()
 {
-	if (window) {
-		gtk_window_present(GTK_WINDOW(window));
-		return;
-	}	
+	if (!window) {	
+		window = gtk_dialog_new();
+		gtk_window_set_transient_for(GTK_WINDOW(window), parent_win);
+
+		gtk_dialog_add_button(GTK_DIALOG(window),
+													GTK_STOCK_HELP,
+													GTK_RESPONSE_HELP);
 	
-	window = gtk_dialog_new();
-	gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (gpAppFrame->oAppCore.window));
+		gtk_dialog_add_button (GTK_DIALOG (window),
+													 GTK_STOCK_CLOSE,
+													 GTK_RESPONSE_CLOSE);
+		gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
+		g_signal_connect(G_OBJECT(window), "response",
+										 G_CALLBACK(response_handler), this);
+		
+		GtkWidget *vbox;
+		vbox = gtk_vbox_new (FALSE, 6);
+		gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
+		
+		GtkWidget *hbox = gtk_hbox_new(false, 3);
+		gtk_box_pack_start(GTK_BOX(vbox),hbox, false, false, 0);
+		
+		wazard_button = gtk_radio_button_new(NULL);
+		GTK_WIDGET_UNSET_FLAGS (wazard_button, GTK_CAN_FOCUS);
+		gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(wazard_button), false);
+		gtk_box_pack_start (GTK_BOX (hbox), wazard_button, false, false, 0);	
+		GtkWidget *hbox1 = gtk_hbox_new(false, 2);
+		gtk_container_add (GTK_CONTAINER (wazard_button), hbox1);
+		GtkWidget *image = gtk_image_new_from_pixbuf(dicts_icon);
+		gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
+		GtkWidget *label = gtk_label_new_with_mnemonic(_("D_ictionaries"));
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), wazard_button);
+		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wazard_button), true);
+		g_signal_connect(G_OBJECT(wazard_button),"toggled", G_CALLBACK(on_wazard_button_toggled), this);
+		
+		GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
+		GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
+		gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
+		gtk_box_pack_start (GTK_BOX (hbox), appendix_button, false, false, 0);	
+		hbox1 = gtk_hbox_new(false, 2);
+		gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
+		image = gtk_image_new_from_pixbuf(tree_dicts_icon);
+		gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
+		label = gtk_label_new_with_mnemonic(_("T_ree dictionaries"));
+		gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+		gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
+		gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);		
+		g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);
+		
+		hbox = gtk_hbox_new (FALSE, 18);
+		gtk_box_pack_start (GTK_BOX (vbox), hbox, true, true, 0);
+		
+		notebook = gtk_notebook_new();
+		gtk_box_pack_start(GTK_BOX(hbox),notebook, true, true, 0);
+		gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), false);
+		
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(false), NULL);
+		gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(true), NULL);
+		
 
-	gtk_dialog_add_button (GTK_DIALOG (window),
-                             	GTK_STOCK_HELP,
-                             	GTK_RESPONSE_HELP);
+		GtkWidget *buttons = create_buttons ();
+		gtk_box_pack_start (GTK_BOX (hbox), buttons, false, false, 0);		
+		
+		label = gtk_label_new_with_mnemonic (_("These settings will take effect the next time you run StarDict."));
+		gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+		g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
+		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+		
+		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox,
+												true, true, 0);
+		
+		
+		gtk_widget_show_all(GTK_DIALOG (window)->vbox);	
 	
-	gtk_dialog_add_button (GTK_DIALOG (window),
-                             	GTK_STOCK_CLOSE,
-                             	GTK_RESPONSE_CLOSE);
-	gtk_dialog_set_default_response (GTK_DIALOG (window), GTK_RESPONSE_CLOSE);
-	g_signal_connect(G_OBJECT (window), "response",
-			 G_CALLBACK (response_handler), this);
-
-	GtkWidget *vbox;
-	vbox = gtk_vbox_new (FALSE, 6);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
-	
-	GtkWidget *hbox = gtk_hbox_new(false, 3);
-	gtk_box_pack_start(GTK_BOX(vbox),hbox, false, false, 0);
-	
-	wazard_button = gtk_radio_button_new(NULL);
-	GTK_WIDGET_UNSET_FLAGS (wazard_button, GTK_CAN_FOCUS);
-	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(wazard_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), wazard_button, false, false, 0);	
-	GtkWidget *hbox1 = gtk_hbox_new(false, 2);
-	gtk_container_add (GTK_CONTAINER (wazard_button), hbox1);
-	GtkWidget *image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.stardict.index_wazard.p[0]);
-	gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
-	GtkWidget *label = gtk_label_new_with_mnemonic(_("D_ictionaries"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), wazard_button);
-	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wazard_button), true);
-	g_signal_connect(G_OBJECT(wazard_button),"toggled", G_CALLBACK(on_wazard_button_toggled), this);
-
-	GtkWidget *appendix_button = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(wazard_button));
-	GTK_WIDGET_UNSET_FLAGS (appendix_button, GTK_CAN_FOCUS);
-	gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(appendix_button), false);
-	gtk_box_pack_start (GTK_BOX (hbox), appendix_button, false, false, 0);	
-	hbox1 = gtk_hbox_new(false, 2);
-	gtk_container_add (GTK_CONTAINER (appendix_button), hbox1);
-	image = gtk_image_new_from_pixbuf(gpAppFrame->oAppSkin.stardict.index_appendix.p[0]);
-	gtk_box_pack_start (GTK_BOX (hbox1), image, FALSE, FALSE, 0);
-	label = gtk_label_new_with_mnemonic(_("T_ree dictionaries"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_box_pack_start (GTK_BOX (hbox1), label, FALSE, FALSE, 0);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), appendix_button);		
-	g_signal_connect(G_OBJECT(appendix_button),"toggled", G_CALLBACK(on_appendix_button_toggled), this);
-
-	hbox = gtk_hbox_new (FALSE, 18);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, true, true, 0);
-
-	notebook = gtk_notebook_new();
-	gtk_box_pack_start(GTK_BOX(hbox),notebook, true, true, 0);
-	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(notebook), false);
-
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(false), NULL);
-	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), create_dict_tree(true), NULL);
-	
-
-	GtkWidget *buttons;
-	buttons = create_buttons ();
-	gtk_box_pack_start (GTK_BOX (hbox), buttons, false, false, 0);		
-
-	label = gtk_label_new_with_mnemonic (_("These settings will take effect the next time you run StarDict."));
-	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	g_object_set (G_OBJECT (label), "xalign", 0.0, NULL);
-	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (window)->vbox), vbox,
-			true, true, 0);
-
-	
-	gtk_widget_show_all (GTK_DIALOG (window)->vbox);	
-	
-	gtk_window_set_title (GTK_WINDOW (window), _("Manage Dictionaries"));
-	//gtk_window_set_resizable (GTK_WINDOW (window), FALSE);
-	gtk_widget_show(window);
-
+		gtk_window_set_title(GTK_WINDOW (window), _("Manage Dictionaries"));	
+	}
+	while (gtk_dialog_run(GTK_DIALOG(window))==GTK_RESPONSE_HELP)
+    ;
+	gtk_widget_hide(GTK_WIDGET(window));
 }
 
 void DictManageDlg::Close()
